﻿package de.maxmaurer.FlashGameCommunicator.Events {
	import flash.events.Event;
	
	/**
	* FlashGameCommunicator - GameEvent
	* <p>This class contains the Event fired every time an Eventchange happens within the 
	* FlashGameCommunicator.</p>
	* <p>Each event carries a different number of parameters which are listed below, which of
	* these parameters contain values depends on the Event-Type as well as the messageType</p>
	* <p>An event listener can be bound to six different types of Events that may happen:</p>
	* <ul>
	* <li><code>CONNECTED</code> The connected-event is fired when the connection to the 
	* FlashGameCommunicator-Server has been established.</li>
	* <li><code>DISCONNECTED</code> The disconnected-event is fired when the connection
	* to the server is ended or when the connection is lost by unknown reason</li>
	* <li><code>MESSAGE</code> Message events are the only events that happend without
	* a prior call to the FlashGameCommunicator-API. Every time a message intended for
	* this client is sent by the server the Message-Event fires. The type of the message
	* (BROADCAST, ROOM, PERSONAL or LOBBY) is stored in the messageType-field.</li>
	* <li><code>SUCCESS</code> Success event describe a successfull execution of the 
	* last command that was sent to the server. Return parameters of this command
	* may be read from this event object.</li>
	* <li><code>FAILED</code> If an action could not be completed successfull this is
	* indicated by a failed-Event.</li>
	* <li><code>KICKED</code> Kicked event appear when the master user of a room leaves
	* his rooms. The room is then removed and all players that where bound to that room
	* are kicked and thrown back to the lobby. This does NOT mean that the client has been
	* disconnected. But he's not anymore bound to any room and has to join again.</li>
	* </ul>
	*/
	public class GameEvent extends Event {
		/** Fired when connection was established */
		public static var CONNECTED:String="flashGameCommunicatorConnected";
		/** Fired when connection was disconnected */
		public static var DISCONNECTED:String="flashGameCommunicatorDisconnected";
		/** Fired when a message is incoming from the server */
		public static var MESSAGE:String="flashGameCommunicatorMessage";
		/** Fired when a user joined the room this user is currently in */
		public static var USER_JOINED:String="flashGameCommunicatorUserJoined";
		/** Fired when a user left the room this user is currently in */
		public static var USER_LEFT:String="flashGameCommunicatorUserLeft";
		/** Fired when a command sent to the server succeeded */
		public static var SUCCESS:String="flashGameCommunicatorSuccess";
		/** Fired when a command failed */
		public static var FAILED:String="flashGameCommunicatorFailed";
		/** Fired when a user has been thrown out of the current room*/
		public static var KICKED:String="flashGameCommunicatorKicked";

		/** Contains the messageType of this Event that may be either one of the 
		* ACTION_... values of the FlashGameCommunicator class, when a server command was issued
		* or one of the MESSAGE_... values of the FlashGameCommunicator class when a Incoming-Message
		* event has been fired. */
		public var messageType:uint;
		/** Contains the users ID after a connection attempt (<code>ACTION_CONNECTING</code>)*/
		public var myId:Number;
		/** Contains the room-id after an <code>ACTION_ROOM_CREATE</code> or an <code>ACTION_ROOM_JOIN</code> */
		public var roomId:Number;
		/** Contains the room-name after an <code>ACTION_ROOM_CREATE</code>, an <code>ACTION_ROOM_JOIN</code> or an <code>ACTION_ROOM_INFORMATION</code>*/
		public var roomName:String;
		/** Contains the maximum room capacity after a <code>ACTION_ROOM_INFORMATION</code> */
		public var roomCapacity:Number;
		/** Contains wether the room is locked or not after an <code>ACTION_ROOM_INFORMATION</code> */
		public var roomLocked:Boolean;
		/** Contains the room size (how many people are already in there) after an <code>ACTION_ROOM_INFORMATION</code> */
		public var roomSize:Number;
		/** Contains the message that has been send after any of the <code>MESSAGE_...</code> eventTypes. */
		public var message:String;
		/** Contains the id of the sender (or 0 for the server itself) after any of the <code>MESSAGE_...</code> eventTypes. */
		public var senderId:Number;
		/** Contains the name of the sender (or something like "Server") after any of the <code>MESSAGE_...</code> eventTypes. */
		public var senderName:String;
		/** Contains the Id of a user when he has joined (<code>INFO_USER_JOINED</code>) or left(<code>INFO_USER_LEFT</code>) the room. */
		public var userId:Number;
		/** Contains the Name of a user when he has joined (<code>INFO_USER_JOINED</code>) or left(<code>INFO_USER_LEFT</code>) the room. */
		public var userName:String;
		/** The dataArray contains a list of values after any server command that issues a list.
		* The number of values returned by the server can be retrieved via <code>dataArray.length</code>.
		* <p>In case of an <code>ACTION_ROOM_LIST</code> the array contains elements with the roomId and 
		* the roomName that can be retrieved via <code>dataArray[0].roomId</code> or <code>dataArray[0].roomName</code>.
		* </p>
		* <p>In case of an <code>ACTION_USER_LIST</code> or an <code>ACTION_USER_LIST_ROOM</code> the 
		* dataArray contains userId and userName of each user that can be access by <code>dataArray[0].userId</code>
		* and <code>dataArray[0].userName</code>.
		* </p>
		*/
		public var dataArray:Array;
		
		/**
		* The GameEvent constructor should never be called externally
		*/		
		function GameEvent(type:String,params:Object):void {
			super(type,false,false);
			myId=params.myId;
			messageType = params.messageType;
			roomId = params.roomId;
			roomName = params.roomName;
			dataArray = params.dataArray;
			roomCapacity = params.roomCapacity;
			roomSize = params.roomSize;
			roomLocked = params.roomLocked;
			senderName = params.senderName;
			senderId = params.senderId;
			message = params.message;
			userId = params.userId;
			userName = params.userName;
		}
	}
}
